"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Async ReduceRight function
 *
 * @export
 * @template T
 * @template R
 * @param {T[]} elements
 * @param {CallBackReduceRight<T, R>} cb
 * @param {R} [initialValue]
 * @returns {Promise<T | R>}
 */
function reduceRight(elements, cb, initialValue) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!elements.length && initialValue === undefined) {
            throw new Error('Reduce of empty array with no initial value');
        }
        let reducedValue;
        const reversedElements = [...elements].reverse();
        if (initialValue === undefined) {
            reducedValue = reversedElements.shift();
        }
        else {
            reducedValue = initialValue;
        }
        for (const [index, element] of reversedElements.entries()) {
            reducedValue = yield cb(reducedValue, element, index, reversedElements);
        }
        return reducedValue;
    });
}
exports.reduceRight = reduceRight;
//# sourceMappingURL=reduce_right.js.map